/*

Written by Phil Wyss

Written 10/5/1

Modification

10/10/1 - Add file name and date in Gene_name
10/10/1 - Add transcription initiation site in Expt
10/10/1 - Set the primary keys for Gene_name and Expt to be AUTOINCREMENT
10/12/1 - Add comments for each table
1/18/2  - Change Gene references to Sequence
1/18/2  - Sequence_name (formerly Gene_name) table no longer contains TI_site
1/18/2  - Expt now specifies TI_site
2/12/2  - Expt_class classifies studied region of sequence
3/9/2   - Classification tables for experiments and oligomers added
3/11/2  - Classification tables for oligomers added
3/16/2  - Classification for sequences added
This file constructs tables for the oligomer analysis programs

Please remember to edit this file if any manual alteration of the
tables are done.

Test or development databases can be created apart from a production
database by CREATE-ing and USE-ing different SQL databases

*/

/*

The sequence to ID mapping table for 9-mers specifically
and can hold a summary of the RF_code locate table, i.e.,
the total # of hits across all experiments listed in the
RF_code_locate table.

The tag field is can be used to flag a record. Not currently used.

*/

CREATE TABLE IF NOT EXISTS RF_code_hits (

	RF_id CHAR(9) NOT NULL,
        RF_9mers CHAR(9) NOT NULL,
	Num_of_hits INT NOT NULL,
	Tag CHAR(1),
	UNIQUE INDEX codehits_id (RF_id),
        UNIQUE INDEX codehits_9mer (RF_9mers)

);

/*

RF_code_locate records locations of ninemers in given region of a sequences
the region is defined by a start/end location defined wrt transciption
initiation site or raw sequence coordinates (which of these is determined
in table Expt) The Exp_num is a foreign key to the Expt table.

The Sequence_id is a foreign key to Sequence_name table.

Each 9mer is identified by its RF_id from RF_code_hits

Location is position of the first base of the 9mer with respect
to the coordinate system of the experiment (Expt table).

Direction defines whether the 9mer is read directly from the sequence
(f for forward) or it was derived a the reverse complement (r for reverse).

The Flag field, if defined, allows discrimination between normal vs.
repetative DNA. The study_seq program sets this to R if lower case 
is used to describe the sequence. The -flag option MUST be used in
study_seq if this field is defined. 

*/

CREATE TABLE IF NOT EXISTS RF_code_locate (

	RF_id CHAR(9) NOT NULL,
	Sequence_id INT NOT NULL,
	Location INT NOT NULL,
	Direction CHAR(1) NOT NULL,
	Flag CHAR(1),
	Exp_num INT NOT NULL,
	INDEX codelocate_id (RF_id),
	INDEX codelocate_gene (Sequence_id),
	INDEX codelocate_exp_num (Exp_num)

);
/*

Sequence_name maps the formal and local names to its accession/version number
The table holds the sequence file name containing the sequence and annotations
The file modification date is stored in UNIX epoch time to be easily compared
to the file statistics to ensure the consistency of the data

An internal id (Sequence_id) is used to link to the RF_code_locate
Expt table

*/

CREATE TABLE IF NOT EXISTS Sequence_name (

	Sequence_id INT AUTO_INCREMENT PRIMARY KEY NOT NULL,
	Sequence_name CHAR(200) NOT NULL,
	Sequence_labname CHAR(20) NOT NULL,
	Sequence_accession CHAR(20) NOT NULL,
	Sequence_file CHAR(255) NOT NULL,
	Sequence_file_date INT UNSIGNED NOT NULL,
	Sequence_position INT NOT NULL,
	Sequence_format CHAR(20) NOT NULL,
	UNIQUE INDEX genename_labname (Sequence_labname),
	UNIQUE INDEX genename_accession (Sequence_accession)

);
/*

The experiment table describes the star/end region of the stated gene
wrt the transcription initiation site (TI_site). It also lists the person (as 
database user id) creating the record and the date that the record was created.

The TI_site field describes the transcription initiation site in raw 
sequence coordinates (first base index = 1).

Start and End locations are wrt to TI_site. TI_site is one if actually
using raw coordinates.

The first and last 9mer found in the studied region is recorded to be
able to check data integrity.

*/

CREATE TABLE IF NOT EXISTS Expt (

	Exp_num INT AUTO_INCREMENT PRIMARY KEY NOT NULL,
	Exp_comment TEXT, 
	Sequence_id INT NOT NULL,
	TI_site INT NOT NULL,
	Start_location INT NOT NULL,
	End_location INT NOT NULL,
	First_mer CHAR(9),
	Last_mer CHAR(9),
	Name_of_person CHAR(30) NOT NULL,
	Date_of_exp DATETIME NOT NULL
	
);

/*

Expt_class allows multiple classifications of a studied subsequence of some
registered sequence. The studied subsequence is described in Expt table

The class is meant to specify a set-theoretical partition and the type
defines a subset in the partition. This means the experiment number,class pair
should only occur once int the Expt_class table.

For example, a class of Content could be defined with types DNA, cDNA, mRNA.

Two subsidiary tables holds the names of the integer-encoded class and type
Unique indices on the name is used to force uniqueness of names.

*/

CREATE TABLE IF NOT EXISTS Expt_class (

	Exp_num INT NOT NULL,
	Class_id INT NOT NULL,
	Type_id INT NOT NULL,
	UNIQUE Exptclass_onceOnly (Exp_num,Class_id),
	INDEX exptClass_class (Class_id),
	INDEX exptClass_type (Type_id)

);

/*

Sequence_class allows multiple classifications of a
registered sequence. The sequence is described in Sequence_name table

The same subsidiary tables as Expt_class are used to hold
names/id of classes and types

*/

CREATE TABLE IF NOT EXISTS Sequence_class (

	Sequence_id INT NOT NULL,
	Class_id INT NOT NULL,
	Type_id INT NOT NULL,
	UNIQUE sequenceClass_onceOnly (Sequence_id,Class_id),
	INDEX sequenceClass (Class_id),
	INDEX sequenceClass_type (Type_id)

);

/*

   classify different oligomers in various ways as we do experiments

*/

CREATE TABLE IF NOT EXISTS Mer_class (

	Mer CHAR(15) PRIMARY KEY NOT NULL,
	Mer_size INT NOT NULL,
	Class_id INT NOT NULL,
	Type_id INT NOT NULL,
	UNIQUE merClass_onceOnly (Mer,Class_id),
	INDEX merClass_class (Class_id),
	INDEX merClass_type (Type_id)

);

CREATE TABLE IF NOT EXISTS Class_names (

	Entity_id INT AUTO_INCREMENT PRIMARY KEY NOT NULL,
	Entity_name CHAR(28) NOT NULL,
	Entity_title CHAR(80),
	UNIQUE INDEX merName_class (Entity_name)

);

CREATE TABLE IF NOT EXISTS Type_names (

	Entity_id INT AUTO_INCREMENT PRIMARY KEY NOT NULL,
	Entity_name CHAR(28) NOT NULL,
	Entity_title CHAR(80),
	UNIQUE INDEX merName_type (Entity_name)

);
