=head1 Package for Experiment object under package mySequence

Written by Phillip J. Wyss, Purdue University, Dept. of Chemistry

Written 10/11/1

6/20/3  add arbitrary flag element to RF_code_locate record if present

=cut

package mySequence::Experiment;

use strict;
require DBI;
require Bio::Seq;
require Util9mer;

use vars qw( $VERSION @ISA @EXPORT @EXPORT_OK );

use Exporter ();
@ISA = qw(Exporter);
@EXPORT_OK = qw();

use vars qw( $lastError );

=head1 Method new - create a new experiment record

Input

	instance or class reference
	database handle reference (connected to a database)
	lab name of the sequence on which to run the experiment
	transcription coordinate of start of experiment
	transcription coordinate of end of experiment
	transcription initiation site (0 if raw sequence coordinates)
		(optional, 0 if not specified)

Result

	An experiment record is created and the same data is stored in
	the hash object that is returned

=cut

sub new {

	my ($class,$dbh,$labname,$start,$end,$first,$last) = @_; # copy parameters into local variables
	my $TIsite = 0; # default is raw sequence coordinates if not specified
	if( scalar @_ > 7) {
		$TIsite = $_[7];
	}

	my $type = ref($class) || $class; # may actually be instance
	my $self = {}; # object hash reference

	my $stmt; # general statement handle to use
	$lastError = ''; # set to no error yet
	my $status = 1; # set to zero if error detected

	my $username; # will contain database use name
	my $exptdate = Util9mer::mkSQLdatetime(time); # date/time of experiment

	my $sequenceID; # id of sequence referred to by labname
#
# get the user name
#
	$stmt = $dbh->prepare("SELECT USER()");
	$stmt->execute();

	if( $stmt->err ) {
		$lastError = $stmt->errstr;
		$status = 0;
	}
	else {
		my $userref = $stmt->fetchrow_arrayref();
		if( $$userref[0] =~ /(.*)(@.*)/ ) {
			$username = $1;
		}
		else {
			$username = $$userref[0];
		}
	}

	$stmt->finish();

#
# now get the id of the sequence to study
#

	if( $status ) {

		$stmt = $dbh->prepare("SELECT Sequence_id from Sequence_name where Sequence_labname = ?");
		if( $stmt ) {
			$stmt->execute($labname);

			if( $stmt->err ) {
				$lastError = $stmt->errstr;
				$status = 0;
			}
			else {
				my $rechash = $stmt->fetchrow_hashref();
				if( $rechash ) {
					$sequenceID = $$rechash{'Sequence_id'};
				}
				else {
					$lastError = "No Sequence Labelled $labname";
					$status = 0;
				}
			}
			$stmt->finish();

		}
		else {
			$lastError = $dbh->errstr;
			$status = 0;
		}

	}

#
# get all other experiments for this sequence
#

	if( $status ) {

		$stmt = $dbh->prepare('SELECT TI_site,Start_location,End_location FROM Expt WHERE Sequence_id = ?');
		if( $stmt ) {
			$stmt->execute($sequenceID);
			if( $stmt->err ) {
				$status = 0;
			}
		}
		else {
			$lastError = $dbh->errstr;
			$status = 0;
		}
	}

#
# compare experiment to previous, insist on no overlap
# $stmt executed in last previous stage of routine
#

	if($status) {

		my $expt = $stmt->fetchrow_arrayref(); # next expt coordinates
		my ($ts,$te) = Util9mer::computeRawCoordinates($TIsite,$start,$end);
		while( $expt && $status) {
			my ($cs,$ce) = Util9mer::computeRawCoordinates(@$expt);
			if( ($cs-$ts)*($ce-$ts) <= 0 || ($cs-$te)*($ce-$te) <= 0|| ($ts-$cs)*($te-$ce) <= 0 ) {	
				$lastError = "Experiment Overlaps Previous Experiment";
				$status = 0;
			}
			$expt = $stmt->fetchrow_arrayref();
		}

	}

#
# create the experiment record if status ok up to now
#

	if( $status ) {
#
# set up the insert statement
#
		 $stmt = $dbh->prepare('INSERT INTO Expt (Sequence_id,Start_Location,End_location,First_mer,Last_mer,TI_site,Name_of_person,Date_of_exp) VALUES (?,?,?,?,?,?,?,?)');
		
		if( !$stmt ) {
			$lastError = $dbh->errstr;
			$status = 0;
			$stmt->finish();
		}

	}
#
# execute insert if no error
#
	if( $status )  {
		$stmt->execute($sequenceID,$start,$end,$first,$last,$TIsite,$username,$exptdate);
		if( $stmt->err ) {
			$lastError = $dbh->errstr;
			$status = 0;
		}
		$stmt->finish();
	}

#
# experiment id of this experiment is the last autonumbered id
#

	if( $status ) {
		
		$stmt = $dbh->prepare('SELECT LAST_INSERT_ID()');
		$stmt->execute();

		if( $stmt->err ) {
			$lastError = $stmt->errstr;
			$status = 0;
		}
		else {
		
			my $idref = $stmt->fetchrow_arrayref();

			if( ! $idref ) {
				$lastError = 'Failed to Retrieve Experiment Number';
				$status = 0;
			}
			else {
				$$self{'Exp_num'} = $$idref[0];
			}
		}

		$stmt->finish();

	}
#
# Copy the rest of the experiment parameters
#
	if( $status )  {

		$$self{'Sequence_id'} = $sequenceID;
		$$self{'Start_location'} = $start;
		$$self{'End_location'} = $end;
		$$self{'TI_site'} = $TIsite;
		$$self{'Name_of_person'} = $username;
		$$self{'Date_of_exp'} = $exptdate;

	}

	if( $status ) {
		return bless $self,$type;
	}
	else {
		return undef;
	}

}
=head1 - method lookup - get an experiment from database

Input

	database handle
	expt number

Result

	values of record's fields are placed in objects key
	(same keys as in new)

=cut

sub lookup {

	my $class = shift; # won't use this, we create a new object
	my $type = ref($class) || $class; # may actually be instance
	my $dbh = shift;
	my $run_num = shift;
	my $self = {};
	my $status = 1; # set to zero on error

#
# lookup run in database
#
	my $stmt = $dbh->prepare('SELECT * FROM Expt WHERE Exp_num = ?');
	if( $dbh->err ) {
		$lastError = $dbh->errstr;
		$status = 0;
	}
	if( $status ) {
		$stmt->execute($run_num);
		if( $stmt->err ) {
			$lastError = $stmt->errstr;
			$status = 0;
			$stmt->finish();
		}
	}

#
# if lookup good, check if found any record, copy data to object
#

	if( $status ) {

		my $record = $stmt->fetchrow_hashref();
		if( $record ) {
			foreach my $key ( keys %$record ) {
				$$self{$key} = $$record{$key};
			}
		}
		else {
			$lastError = 'No Such Experiment Number';
			$status = 0;
		}
		$stmt->finish();

	}

	if( $status ) {
		$stmt->finish();
		return bless $self,$type;
	}
	else {
		return undef;
	}

}
=head1 - Instance Method comment - update the experiment's comment

Input

	instance reference
	database handle reference
	text of comment

Results

	the comment field of the Expt table is updated for the
	record whose experiment number is found in the instance hash

=cut

sub comment {

	my ($self,$dbh,$comment) = @_;
	my $status = 1; # set to 1 on error
	my $lastError = ''; # no error yet

	my $stmt = $dbh->prepare('UPDATE Expt SET Exp_comment = ? WHERE Exp_num = ?');

	if( ! $stmt ) {
		$lastError = $dbh->errstr;
		$status = 0;
	}

	if( $status ) {
		$stmt->execute($comment,$self->{'Exp_num'});
		if( $stmt->err ) {
			$lastError = $stmt->errstr;
			$status = 0;
		}
		$stmt->finish();
	}

	return $status;

}

=head1 - Instance Method recordLocation - record location of 9mer on sequence

Input

	instance reference
	database handle reference
	The RF id of the 9mer
	The transcription coordinate where the 9mer occurs wrt TI_site
	Flag indicating which direction it was located
	Arbitrary single character flag (meaning is external to this routine) optional

Results

	A record is inserted into the RF_code_locate table
	Zero is returned on failure, non-zero (true) on success

=cut

sub recordLocation {

	my $self = shift;
	my $dbh = shift;
	my $RFid = shift;
	my $location = shift;
	my $direction = shift;
	my $flag;
	if( @_ ) {
		$flag = shift;
	}
	my $status = 1; # set to zero on error
	my $lastError = ''; # no error yet

	my $stmt; # prepare SQL statement
	if( exists($$self{'INSERT_LOC_STMT'}) ) {
		$stmt = $$self{'INSERT_LOC_STMT'};
	}
	else {
		if( $flag ) {
			$stmt = $dbh->prepare('INSERT INTO RF_code_locate VALUES (?,?,?,?,?,?)');
		}
		else {
			$stmt = $dbh->prepare('INSERT INTO RF_code_locate VALUES (?,?,?,?,?)');
		}
	}

	if( ! $stmt ) {
		$lastError = $dbh->errstr;
		$status = 0;
	}

	if( $status ) {
		$$self{'INSERT_LOC_STMT'} = $stmt;
		if( defined $flag ) {
			$stmt->execute($RFid,$$self{'Sequence_id'},$location,$direction,$flag,$$self{'Exp_num'});
		}
		else {
			$stmt->execute($RFid,$$self{'Sequence_id'},$location,$direction,$$self{'Exp_num'});
		}
		if( $stmt->err ) {
			$lastError = $stmt->errstr;
			$status = 0;
		}

	}

	return $status;

}
=head1 classify - instance method to classify an experiment

Input

	instance of experiment object
	database handle to database in which to record class information
	name of class
	name of type

Results

	The class code and type code are looked up in the database
	A record of experiment #,class #,type # are recorded in
	Expt_class table

Return Value

	0 on failure, 1 on success

=cut

sub classify {

	my $self = shift;
	my $dbh = shift;
	my $class = shift;
	my $type = shift;

	my $result = 0; # will set to success if all db actions successful
	my $stmt; # ref to statement handle

#
# make sure names are known
#

	my ($classid,$typeid) =  Util9mer::findEntity($dbh,$class,'Class_names',$type,'Type_names');

	if( defined($classid) && defined($typeid) ) {
		$stmt = $dbh->prepare('INSERT INTO Expt_class VALUES (?,?,?)');
		if( !$stmt->err ) {
			$stmt->execute($self->{'Exp_num'},$classid,$typeid);
		}
		if( $stmt->err ) {
			$lastError = $stmt->errstr;
		}
		else {
			$result = 1;
		}
	}
	else {
		$lastError = 'No Such Class or Type';
	}

	return $result;

}
=head1 remove - remove this experiment from the database

Input

	instance of experiment object
	database handle to database from which to remove run

Results

	The experiment, its count records and classifications are deleted

Return Value

	0 on failure, 1 on success

=cut

sub remove {

	my $self = shift;
	my $dbh = shift;

	my $result = 0; # will set to success if all db actions successful
	Util9mer::execSQLstmt($dbh,"DELETE FROM Expt_class WHERE Exp_num = ?",$$self{'Exp_num'});
	if( $Util9mer::lastError ) {
		$lastError = $Util9mer::lastError;
	}
	else {
		Util9mer::execSQLstmt($dbh,'DELETE FROM RF_code_locate WHERE Exp_num = ?',$$self{'Exp_num'});
		if( $Util9mer::lastError ) {
			$lastError = $Util9mer::lastError;
		}
		else {
			Util9mer::execSQLstmt($dbh,'DELETE FROM Expt WHERE Exp_num = ?',$$self{'Exp_num'});
			if( $Util9mer::lastError ) {
				$lastError = $Util9mer::lastError;
			}
			else {
				$result = 1;
			}
		}
	}

	return $result;

}

#
# the destructor cleans up the insert location sql statement
#

sub DESTROY {

	my $self = shift;
	if( exists($$self{'INSERT_LOC_STMT'}) ) {
		$$self{'INSERT_LOC_STMT'}->finish();
	}
}

return 1;


